@@ -1,15 +1,13 @@
 use super::Vec3;
-#[cfg(not(target_arch = "spirv"))]
-use super::{Mat4, Quat};
 
 /// A 3-dimensional axis-aligned bounding box
 #[derive(Clone, Copy, Default, PartialEq)]
 #[cfg_attr(feature = "with_serde", derive(serde::Serialize, serde::Deserialize))]
 #[cfg_attr(feature = "with_speedy", derive(speedy::Writable, speedy::Readable))]
 pub struct BoundingBox {
-    /// Bounding box minimum
+    /// Bounding box minimum (inclusive).
     pub min: Vec3,
-    /// Bounding box maximum
+    /// Bounding box maximum (inclusive).
     pub max: Vec3,
 }
 
@@ -22,17 +20,44 @@ impl core::fmt::Debug for BoundingBox {
 
 #[allow(unused)]
 impl BoundingBox {
-    /// Create a bounding box from a minimum and maximum position
+    /// A [`BoundingBox`] that only contains [`Vec3::ZERO`].
+    pub const ZERO: BoundingBox = BoundingBox {
+        min: Vec3::ZERO,
+        max: Vec3::ZERO,
+    };
+
+    /// A [`BoundingBox`] that contains no points.
+    ///
+    /// This is useful as the seed for bounding bounding boxes.
+    #[inline]
+    pub fn nothing() -> Self {
+        Self {
+            min: Vec3::splat(core::f32::INFINITY),
+            max: Vec3::splat(core::f32::NEG_INFINITY),
+        }
+    }
+
+    /// A [`BoundingBox`] that contains every point.
+    #[inline]
+    pub fn everything() -> Self {
+        Self {
+            min: Vec3::splat(core::f32::NEG_INFINITY),
+            max: Vec3::splat(core::f32::INFINITY),
+        }
+    }
+
+    /// Create a bounding box from a minimum and maximum position.
+    #[inline]
     pub fn from_min_max(min: Vec3, max: Vec3) -> Self {
         Self { min, max }
     }
 
-    /// Create a bounding box from a center position and a size
+    /// Create a bounding box from a center position and a size.
     pub fn from_center_size(center: Vec3, size: Vec3) -> Self {
         Self::from_min_max(center - 0.5 * size, center + 0.5 * size)
     }
 
-    /// Create a bounding box from an iterator of points that the bounding box will cover
+    /// Create a bounding box from an iterator of points that the bounding box will cover.
     pub fn from_points(points: impl Iterator<Item = Vec3>) -> Self {
         let mut bb = Self::nothing();
         for p in points {
@@ -41,64 +66,68 @@ impl BoundingBox {
         bb
     }
 
-    pub fn nothing() -> Self {
-        Self {
-            min: Vec3::splat(core::f32::INFINITY),
-            max: Vec3::splat(core::f32::NEG_INFINITY),
-        }
-    }
-
-    pub fn everything() -> Self {
-        Self {
-            min: Vec3::splat(core::f32::NEG_INFINITY),
-            max: Vec3::splat(core::f32::INFINITY),
-        }
-    }
-
-    /// Returns the center point of the bounding box
+    /// Returns the center point of the bounding box.
+    #[inline]
     pub fn center(&self) -> Vec3 {
         (self.min + self.max) * 0.5
     }
 
-    /// Returns the 3D axis size of the bounding box
+    /// Returns the 3D axis size of the bounding box.
+    #[inline]
     pub fn size(&self) -> Vec3 {
         self.max - self.min
     }
 
-    /// Returns half the size, kind of the radius. Common operation so gets its own function.
+    /// Returns half the size (similar to a radius).
+    #[inline]
     pub fn half_size(&self) -> Vec3 {
         0.5 * (self.max - self.min)
     }
 
-    /// Only correct for positively sized boxes
+    /// Only correct for positively sized boxes.
     pub fn volume(&self) -> f32 {
         let s = self.size();
         s.x * s.y * s.z
     }
 
-    /// True if and only if there is no point for which `bb.contains(point)` is true.
-    /// The opposite of `is_something()`.
-    pub fn is_nothing(&self) -> bool {
-        self.max.x < self.min.x || self.max.y < self.min.y || self.max.z < self.min.z
-    }
-
     /// True if and only if there is at least one point for which `bb.contains(point)` is true.
+    ///
+    /// Will return `true` if [`Self::min`] == [`Self::max`].
     /// The opposite of `is_nothing()`.
     pub fn is_something(&self) -> bool {
         self.min.x <= self.max.x && self.min.y <= self.max.y && self.min.z <= self.max.z
     }
 
-    /// true if min == max
+    /// True if and only if there is no point for which `bb.contains(point)` is true.
+    ///
+    /// The opposite of `is_something()`.
+    pub fn is_nothing(&self) -> bool {
+        self.max.x < self.min.x || self.max.y < self.min.y || self.max.z < self.min.z
+    }
+
+    /// True if this box contains exactly one point.
+    ///
+    /// `true` if [`Self::min`] == [`Self::max`].
+    #[inline]
     pub fn is_point(&self) -> bool {
         self.min == self.max
     }
 
-    /// Checks if the bounding box only contains finite numbers
+    /// Returns `true` if, and only if, all elements are finite.
+    ///
+    /// If any element is either `NaN`, positive or negative infinity, this will return `false`.
+    #[inline]
     pub fn is_finite(&self) -> bool {
         self.min.is_finite() && self.max.is_finite()
     }
 
-    /// The eight corners of this bounding box
+    /// Returns `true` if any elements are `NaN`.
+    #[inline]
+    pub fn is_nan(&self) -> bool {
+        self.min.is_nan() || self.max.is_nan()
+    }
+
+    /// The eight corners of this bounding box.
     pub fn corners(&self) -> [Vec3; 8] {
         [
             Vec3::new(self.min.x, self.min.y, self.min.z),
@@ -112,6 +141,32 @@ impl BoundingBox {
         ]
     }
 
+    /// The twelve edges of this bounding box.
+    pub fn edges(&self) -> [[Vec3; 2]; 12] {
+        use glam::vec3;
+        let a = self.min;
+        let b = self.max;
+        [
+            // along X:
+            [vec3(a.x, a.y, a.z), vec3(b.x, a.y, a.z)],
+            [vec3(a.x, b.y, a.z), vec3(b.x, b.y, a.z)],
+            [vec3(a.x, a.y, b.z), vec3(b.x, a.y, b.z)],
+            [vec3(a.x, b.y, b.z), vec3(b.x, b.y, b.z)],
+            // along Y:
+            [vec3(a.x, a.y, a.z), vec3(a.x, b.y, a.z)],
+            [vec3(b.x, a.y, a.z), vec3(b.x, b.y, a.z)],
+            [vec3(a.x, a.y, b.z), vec3(a.x, b.y, b.z)],
+            [vec3(b.x, a.y, b.z), vec3(b.x, b.y, b.z)],
+            // along Z:
+            [vec3(a.x, a.y, a.z), vec3(a.x, a.y, b.z)],
+            [vec3(b.x, a.y, a.z), vec3(b.x, a.y, b.z)],
+            [vec3(a.x, b.y, a.z), vec3(a.x, b.y, b.z)],
+            [vec3(b.x, b.y, a.z), vec3(b.x, b.y, b.z)],
+        ]
+    }
+
+    /// Enlarge the box to include this point.
+    #[inline]
     pub fn extend(&mut self, pos: Vec3) {
         self.min = self.min.min(pos);
         self.max = self.max.max(pos);
@@ -124,6 +179,8 @@ impl BoundingBox {
         self
     }
 
+    /// Returns the smallest volume that is covered by both `self` and `other`,
+    /// or [`Self::nothing`] if the boxes are disjoint.
     #[must_use]
     pub fn intersection(mut self, other: Self) -> Self {
         let intersection = Self {
@@ -137,7 +194,7 @@ impl BoundingBox {
         }
     }
 
-    /// Returns `true` if the point is within or on the edge of the box.
+    /// Returns `true` if the point is within (or on the edge of) the box.
     #[must_use]
     pub fn contains(&self, point: Vec3) -> bool {
         (self.min.x <= point.x && point.x <= self.max.x)
@@ -145,7 +202,7 @@ impl BoundingBox {
             && (self.min.z <= point.z && point.z <= self.max.z)
     }
 
-    /// Expand with this much padding on each side
+    /// Expand with this much padding on each side.
     #[must_use]
     pub fn expanded(&self, padding: Vec3) -> Self {
         Self {
@@ -154,6 +211,7 @@ impl BoundingBox {
         }
     }
 
+    /// Translate (move) the box by this much.
     #[must_use]
     pub fn translated(&self, translation: Vec3) -> Self {
         Self {
@@ -162,9 +220,10 @@ impl BoundingBox {
         }
     }
 
+    /// Return a bounding box that contains this box after it has been rotated around [`Vec3::ZERO`].
     #[must_use]
     #[cfg(not(target_arch = "spirv"))]
-    pub fn rotated_around_origin(&self, q: &Quat) -> Self {
+    pub fn rotated_around_origin(&self, q: &crate::Quat) -> Self {
         if self.is_nothing() {
             Self::nothing()
         } else {
@@ -173,9 +232,21 @@ impl BoundingBox {
         }
     }
 
+    /// Return a bounding box that contains this box after it has been transformed.
     #[must_use]
     #[cfg(not(target_arch = "spirv"))]
-    pub fn transform(&self, m: &Mat4) -> Self {
+    pub fn transform_iso(&self, m: &crate::IsoTransform) -> Self {
+        if self.is_nothing() {
+            Self::nothing()
+        } else {
+            Self::from_points(self.corners().iter().map(|&c| m.transform_point3(c)))
+        }
+    }
+
+    /// Return a bounding box that contains this box after it has been transformed.
+    #[must_use]
+    #[cfg(not(target_arch = "spirv"))]
+    pub fn transform_affine3(&self, m: &crate::Affine3A) -> Self {
         if self.is_nothing() {
             Self::nothing()
         } else {
@@ -186,13 +257,15 @@ impl BoundingBox {
 
 #[test]
 fn test_bounding_box() {
+    use crate::{Affine3A, Quat};
     let bb = BoundingBox::from_min_max(Vec3::ZERO, Vec3::ZERO);
     assert!(bb.contains(Vec3::ZERO));
     assert!(bb.is_something());
     assert!(!bb.is_nothing());
-    let bb_rotated = bb.transform(&Mat4::from_quat(Quat::from_axis_angle(Vec3::X, 0.5)));
+    let bb_rotated =
+        bb.transform_affine3(&Affine3A::from_quat(Quat::from_axis_angle(Vec3::X, 0.5)));
     assert_eq!(bb, bb_rotated);
-    let bb_translated = bb.transform(&Mat4::from_translation(Vec3::new(2.0, 3.0, 5.0)));
+    let bb_translated = bb.transform_affine3(&Affine3A::from_translation(Vec3::new(2.0, 3.0, 5.0)));
     assert!(bb_translated.is_something());
     assert!(bb_translated.contains(Vec3::new(2.0, 3.0, 5.0)));
 }
