@@ -17,15 +17,6 @@ pub trait QuatExt: Sized {
     ///
     /// This is generally what you want to use to construct a view-rotation when +Z is forward and +Y is up.
     fn rotate_positive_z_towards(forward: Vec3, up: Vec3) -> Option<Quat>;
-
-    /// Return a Quaternion that rotates +Z to the `forward` direction,
-    /// using `up` to control roll, so that +Y will approximately point in the `up` direction.
-    ///
-    /// Will return [`None`] if either argument is zero, non-finite, or if they are colinear.
-    #[deprecated = "Use either rotate_positive_z_towards or rotate_negative_z_towards"]
-    fn from_look_rotation(forward: Vec3, up: Vec3) -> Option<Quat> {
-        Self::rotate_positive_z_towards(forward, up)
-    }
 }
 
 impl QuatExt for Quat {
