@@ -5,8 +5,9 @@ use crate::Vec3;
 /// Any point on the ray can be found through the formula `origin + t * dir`,
 /// where t is a non-negative floating point value, which represents the distance
 /// along the ray.
-#[derive(Clone, Copy, PartialEq)]
+#[derive(Clone, Copy, Default, PartialEq)]
 #[cfg_attr(feature = "with_serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "with_speedy", derive(speedy::Writable, speedy::Readable))]
 pub struct Ray3 {
     /// Start of the ray
     pub origin: Vec3,
@@ -15,6 +16,12 @@ pub struct Ray3 {
 }
 
 impl Ray3 {
+    /// An invalid ray, starting at the origin and going nowhere.
+    pub const ZERO: Self = Self {
+        origin: Vec3::ZERO,
+        dir: Vec3::ZERO,
+    };
+
     /// `dir` should be normalized
     #[inline(always)]
     pub fn from_origin_dir(origin: Vec3, dir: Vec3) -> Self {
@@ -119,6 +126,26 @@ impl core::ops::Mul<Ray3> for crate::IsoTransform {
     }
 }
 
+impl core::ops::Mul<Ray3> for glam::Affine3A {
+    type Output = Ray3;
+    fn mul(self, rhs: Ray3) -> Ray3 {
+        Ray3 {
+            origin: self.transform_point3(rhs.origin),
+            dir: self.transform_vector3(rhs.dir),
+        }
+    }
+}
+
+impl core::ops::Mul<Ray3> for glam::Mat4 {
+    type Output = Ray3;
+    fn mul(self, rhs: Ray3) -> Ray3 {
+        Ray3 {
+            origin: self.transform_point3(rhs.origin),
+            dir: self.transform_vector3(rhs.dir),
+        }
+    }
+}
+
 #[cfg(not(target_arch = "spirv"))]
 impl std::fmt::Debug for Ray3 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
