@@ -8,6 +8,15 @@ use num_traits::Float;
 ///
 /// Adds additional functionality to [`Vec2`] that [`glam`] doesn't have.
 pub trait Vec2Ext {
+    /// `x: cos(angle), y: sin(angle)` (in radians)
+    fn from_angle(angle: f32) -> Self;
+
+    /// Angle of the vector: `y.atan2(x)`
+    fn angle(&self) -> f32;
+
+    /// For element `i` of `self`, return `v[i].trunc()`
+    fn trunc(self) -> Self;
+
     /// For element `i` of the return value, returns 0.0 if `v[i] < edge[i]` and 1.0 otherwise.
     fn step(self, edge: Self) -> Self;
 
@@ -22,6 +31,19 @@ pub trait Vec2Ext {
 }
 
 impl Vec2Ext for Vec2 {
+    #[inline]
+    fn trunc(self) -> Self {
+        vec2(self.x.trunc(), self.y.trunc())
+    }
+
+    fn from_angle(angle: f32) -> Self {
+        Self::new(angle.cos(), angle.sin())
+    }
+
+    fn angle(&self) -> f32 {
+        self.y.atan2(self.x)
+    }
+
     fn step(self, edge: Self) -> Self {
         vec2(self.x.step(edge.x), self.y.step(edge.y))
     }
